/* Interface for ButtonCell class
 *
 * Copyright (C)  1993  The Board of Trustees of  
 * The Leland Stanford Junior University.  All Rights Reserved.
 *
 * Authors: Scott Francis, Fred Harris, Paul Kunz, Imran Qureshi,
 * and Libing Wang.
 *
 * This file is part of an Objective-C class library for a window system
 *
 * ButtonCell.h,v 1.36 1994/12/19 22:40:39 pfkeb Exp
 */

 
#ifndef _ButtonCell_h_
#define _ButtonCell_h_

#include "ActionCell.h"
#include "sysparams.h"

#define NXBUTTON 0
#define NXRADIO 1
#define NXSWITCH 2

/* Button Types */
#define NX_MOMENTARYPUSH    0
#define NX_PUSHONPUSHOFF    1
#define NX_TOGGLE           2
#define NX_SWITCH           3
#define NX_RADIOBUTTON      4
#define NX_MOMENTARYCHANGE  5

@interface ButtonCell:ActionCell
{
    int 	popup;	/* is this a popupshell buttoncell */
    char	*altContents;
    char	*icon;
    char	*altIcon;
    char        *iconFile;
    char	*altIconFile;
    int		widgetType;  /* 0=button, 1=NXRadio, 2=NXSwitch */
    int		radioBehavior;
    unsigned long	backgroundColor;
//    unsigned short	keyEquivalent;
/*
    union _icon    {
        struct _bmap    {
            id      normal;
            id      alternate;
        }
        struct _ke      {
            id      font;
            float   descent;
        }
    }  icon;
    id sound;
*/    
    struct _bcFlags1    {
        unsigned int     pushIn:1;          /* these aren't used right now...*/
        unsigned int     changeContents:1;
        unsigned int     changeBackground:1;
        unsigned int     changeGray:1;
        unsigned int     lightByContents:1;
        unsigned int     lightByBackground:1;
        unsigned int     lightByGray:1;
        unsigned int     hasAlpha:1;
        unsigned int     bordered:1;
        unsigned int     iconOverlaps:1;
        unsigned int     horizontal:1;
        unsigned int     bottomOrLeft:1;
        unsigned int     iconAndText:1;
        unsigned int     lastState:1;
        unsigned int     iconSizeDiff:1;
        unsigned int     iconIsKeyEquivalent:1;
    }          bcFlags1;
    struct _bcFlags2    {
        unsigned int     keyEquivalent:8;  /* these aren't used right now...*/
        unsigned int     transparent:1;
        unsigned int     _padding:7;
    }          bcFlags2;
}


- init;
- copyFromZone:(NSZone *)zone;
- setIcon:(const char *)iconName;
- (const char *)icon;
- setTitle:(const char *)aString;
- (const char *)title;
- setKeyEquivalent:(unsigned short)charCode;
- _setMode:(int)aMode;
- setState:(int)value;
- performClick:sender;
- read:(TypedStream*)typedStream;
- (int)_radioType;
@end

@interface ButtonCell(WidgetSet)
- _init;
- _setStringValue:(const char *)aString;
- _setState:(int)value;
- _setPopUp;
- _addCallback;
- _managedBy:parent;
@end

#endif
