/* appkit.c
 *
 * various generic c-routines for appkit
 *
 * Copyright (C)  1993  The Board of Trustees of  
 * The Leland Stanford Junior University.  All Rights Reserved.
 *
 * Authors: Scott Francis, Adam Fedor (CU, Boulder), and Paul Kunz
 *
 * This file is part of an Objective-C class library for a window system
 *
   appkit.c,v 1.2 1995/02/22 18:28:28 pfkeb Exp
 */

#include <appkit/appkit.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>

char *NXGetTempFilename( char *name, int pos)
    /* description:
       This function creates a unique file name by altering the "name" argument it is passed.
       It replaces the six character starting at the "pos"th position within "name" with digits
       it generates; it then checks whether the file name is unique.  If it is, the file name is
       returned; if not, different digits are tried until a unique name is found.  It is similar
       to the standard C function mktemp(), except that it can leave suffixed intact since you
       specify the location of the character that get replaced.
       */
    
{
    FILE *file;
    
    int random,tries=100;
    
    if (pos < 0 || pos > (strlen(name)-6)) return NULL;

    random = (int) time(NULL);
    do {
        if (!tries--) return NULL;
        random %= 1000000;
        sprintf(&name[pos],"%06d",random);
        file = fopen(name,"r");
        if (file != NULL) fclose(file);
        random++;
    } while (file != NULL);

    return name;
}
