/* CGTMatrix - Object encapsulation of coordinate transforms
 *
 * Authors: Adam Fedor
 *
 * This file is part of an Objective-C class library for X-window system
 *
 * CGTMatrix.m,v 1.1 1995/04/17 22:46:02 fedor Exp
 */

#include "CGTMatrix.h"

@implementation CGTMatrix

- translate: (NXPoint)point
{
    cgt = concatCGT(cgt, composeCGT(NULL, 0, &point));
    return self;
}

- rotate: (NXCoord)degrees
{
    cgt = concatCGT(cgt, composeCGT(NULL, degrees, NULL));
    return self;
}

- scale: (NXSize)scale
{
    cgt = concatCGT(cgt, composeCGT(&scale, 0, NULL));
    return self;
}

- scale:(float)x :(float)y
{
    NXSize scale;
    
    scale.width = x;
    scale.height = y;
    return [self scale:scale];
}

- concatMatrix: (cgt_t)acgt
{
    cgt = concatCGT(cgt, acgt);
    return self;
}

- invert
{
    invertCGT(cgt);
    return self;
}

- (NXPoint) offset
{
    return offsetFromCGT(cgt);
}

- (NXSize) scale
{
    return scaleFromCGT(cgt);
}

- (NXCoord) rotation
{
    return rotationFromCGT(cgt);
}

- (cgt_t) currentMatrix
{
    return cgt;
}

- (NXPoint) transformPoint: (NXPoint)aPoint
{
    return applyCGT(aPoint, cgt);
}

- (NXRect) transformRect: (NXRect)aRect
{
    NXPoint max;

    max.x = NX_MAXX(&aRect);
    max.y = NX_MAXY(&aRect);
    aRect.origin = applyCGT(aRect.origin, cgt);
    max = applyCGT(max, cgt);
    NX_WIDTH(&aRect) = max.x - NX_X(&aRect);
    NX_HEIGHT(&aRect) = max.y - NX_Y(&aRect);
    return aRect;
}

@end
