/* Interface for XtGState class
 *
 * Copyright (C)  1995  The Board of Trustees of  
 * The Leland Stanford Junior University.  All Rights Reserved.
 *
 * Authors: Paul Kunz
 *
 * This file is part of an Objective-C class library for X-window system
 *
 * XtGState.h,v 1.7 1995/07/03 18:13:52 pfkeb Exp
 */

#ifndef __XtGState_h_
#define __XtGState_h_

#define Object X11Object        /* To avoid problems with Objective C */
#define Window X11Window 
#define String X11String
#include <X11/Intrinsic.h>
#undef Object
#undef Window
#undef String

#include <objc/Object.h>
#include <dpsclient/xwfriends.h>
#include <dpsclient/cgt.h>
#include <appkit/graphics.h>

@class View;

@interface XtGState : Object
{
    cgt_t	cgt;
    NXRect	cliprect;
    GC		xgcntxt;
    Widget	widget;
    Drawable	draw;
    View        *view;
    BOOL        sharedGC;
    BOOL        update;    /* indicates draw to cache but not screen yet */
    BOOL        didDraw;   /* indicates draw to screen */
}

+ initialize;
+ alloc;
+ allocFromZone:(NSZone *)zone;
+ removeFromList:(XtGState *)aGState;
+ (int)objectToNum:(XtGState *)aGState;
+ numToObject:(int)aNum;

- initFromSuper:(XtGState *)superGState withView:(View *)aView;
- free;
- setGraphicContext:(GC)context shared:(BOOL)flag;
- (GC)graphicContext;
- (BOOL)hasGraphicContext;
- copyGraphicContext: (XWContext) context;
- createGraphicContext:(XWContext) context;

- setGray:(float)gray;
- translate:(const NXPoint *)point;
- rotate:(float)degrees;
- scale:(NXSize)scale;
- concatCGT:(cgt_t)acgt;
- (NXPoint)offset;
- (NXSize)scale;
- (float)rotation;
- (cgt_t)currentCGT;

- getClipRect:(NXRect *)aRect;
- setNeedsDisplay:(BOOL)flag;
- (BOOL)needsDisplay;
- (BOOL)didDraw;
- (BOOL)hasWidget;
- (Drawable)drawable;
- (BOOL)hasDrawable;

- convertPointToDrawable:(NXPoint *)aPoint;
- convertRectToDrawable:(NXRect *)aRect;
- setDidUpdate;
- copyDrawable:(Drawable)source fromRect:( NXRect *)aRect;
- copyDrawable:(Drawable)source fromRect:(NXRect *)aRect 
			         toPoint:(NXPoint *)aPoint;
- copyImage:(XImage *)image ofSize:(NXSize *)size;
- rectFill:(const NXRect *)aRect;
- (void)flushRect:(NXRect *)frame;
- (void)flushRectIfNeeded:(NXRect *)frame;
- (BOOL) hasSameWidgetAs:(View *)aView;

@end

#endif /* __XtGState_h */
